@@ -243,12 +243,13 @@ clear_step_data <- function(id, .session) {
   session$output[[name]] <- function() value
 }
 
-gui_update_filter_class <- function(step_id, filter_id, show, class, session) {
+gui_update_filter_class <- function(step_id, filter_id, show, class, session, child = ".cb_filter_content") {
   session$sendCustomMessage(
     "update_filter_class",
     list(
       step_id = step_id, filter_id = filter_id,
-      show = show, ns_prefix = session$ns(""), class = class
+      show = show, ns_prefix = session$ns(""),
+      class = class, child = child
     )
   )
 }
@@ -264,23 +265,29 @@ update_filter_gui <- function(cohort, step_id, filter_id, update, reset, session
   if (("multi_input" %in% update) && filter$gui$multi_input) {
     update <- c(update, "input")
   }
+  if ("force_input" %in% update) {
+    update <- c(update, "input")
+  }
 
   if ("input" %in% update) {
-    updated_input <- TRUE
-
     filter$gui$update(
       session,
       sf_id(step_id, filter_id),
       cohort,
       reset = reset
     )
+
+    updated_input <- TRUE
   }
   if ("plot" %in% update) {
     show <- TRUE
     if (!cohort$get_cache(step_id, filter_id, state = "pre")$n_data) {
       show <- FALSE
     }
-    gui_update_filter_class(step_id, filter_id, show, "cb_no_data", session)
+    gui_update_filter_class(
+      step_id, filter_id, show, "cb_no_data", session,
+      child = ".cb_filter_content .cb_no_data_placeholder"
+    )
     show_feedback <- if_null_default(
       filter$get_params("feedback"),
       cohort$attributes$feedback
@@ -364,21 +371,36 @@ update_next_step <- function(cohort, step_id, reset, session) {
   }
 }
 
+overwrite_input_handler <- list(
+  "sw.airdatepicker" = function() ...
+)
+
 input_val_handler <- function(val, binding) {
   handler <- NULL
-  if (length(binding) && binding != "" && !is.na(binding)) {
+  if (!length(binding)) {
+    binding <- ""
+  }
+  if (binding %in% names(overwrite_input_handler)) {
+    handler <- overwrite_input_handler
+  } else if (binding != "" && !is.na(binding)) {
     handler <- `%:::%`("shiny", "inputHandlers")$get(binding)
   }
   if (!is.null(handler)) {
-    return(handler(val))
-  } else if (is.list(val)) {
+    val <- handler(val)
+    if ("air.datetime" %in% binding) {
+      if (is.null(val)) return(c(Inf, -Inf))
+      if (length(val) == 1) return(c(val, val))
+    }
+    return(val)
+  }
+  if (is.list(val)) {
     if (is.null(names(val))) {
       return(unlist(val, recursive = TRUE))
     }
     return(val)
-  } else {
-    return(val)
   }
+
+  val
 }
 
 convert_input_value <- function(changed_input, step_id, filter_id, cohort, update_active) {
@@ -390,6 +412,7 @@ convert_input_value <- function(changed_input, step_id, filter_id, cohort, updat
   update_keep_na <- changed_input$input_name == "keep_na"
   changed_input$input_name <- NULL
   changed_input$input_value <- NULL
+  changed_input$update <- NULL
 
   return(changed_input)
 }
@@ -398,6 +421,7 @@ gui_update_filter <- function(cohort, changed_input, session) {
 
   run_on_request <- !is_none(cohort$attributes$run_button)
   update_active <- changed_input$input_name == "active"
+  update <- changed_input$update
 
   step_id <- changed_input$step_id
   filter_id <- changed_input$filter_id
@@ -431,7 +455,7 @@ gui_update_filter <- function(cohort, changed_input, session) {
   )
   post_stats_visible <- "post" %in% filter_stats
   if (run_update) {
-    update <- c("plot", "multi_input")
+    update <- c(update, "plot", "multi_input")
     if (!run_on_request && post_stats_visible) {
       update <- c(update, "post_input")
     }
@@ -474,6 +498,45 @@ insert_filter <- function(step_id, filter_id, cohort, session) {
   return(invisible(TRUE))
 }
 
+gui_add_step_filter <- function(step_id, filter_id, cohort, session) {
+
+  ns <- session$ns
+  step_filter_id <- sf_id(step_id, filter_id)
+
+  cohort$modify(function(public, private) {
+    private$steps[[step_id]]$filters[[filter_id]] <- attach_filter_gui(
+      private$steps[[step_id]]$filters[[filter_id]]
+    )
+  })
+  filter <- cohort$get_filter(step_id, filter_id)
+  step_filter_target_id <- .filter_position(cohort$get_source(), step_id = step_id, filter = filter, ns = ns)
+
+  filter_ui <- .render_filter(filter, step_id, cohort, ns)
+
+  shiny::insertUI(
+    selector = step_filter_target_id,
+    where = "beforeEnd",
+    ui = filter_ui,
+    immediate = TRUE,
+    session = session
+  )
+}
+
+gui_rm_step_filter <- function(step_id, filter_id, cohort, session) {
+  ns <- session$ns
+  step_filter_id <- sf_id(step_id, filter_id)
+  session$userData$rendered_filters <- setdiff(
+    session$userData$rendered_filters,
+    ns(step_filter_id)
+  )
+
+  shiny::removeUI(
+    selector = glue::glue("#{ns(step_filter_id)}"),
+    immediate = TRUE,
+    session = session
+  )
+}
+
 gui_rm_step <- function(cohort, changed_input, session) {
   ns <- session$ns
 
@@ -487,6 +550,125 @@ gui_rm_step <- function(cohort, changed_input, session) {
   session$sendCustomMessage("post_rm_step_action", list(id = changed_input$step_id, ns_prefix = ns("")))
 }
 
+gui_manage_step_modal <- function(cohort, changed_input, session) {
+  ns <- session$ns
+
+  print_state("manage_step_modal", changed_input)
+  input_state("manage_step_modal", changed_input)
+
+  available_filters <- cohort$attributes$available_filters
+
+  if (length(available_filters) == 0) {
+    warning_nl("`available_filters` was not defined, configure step will not be working. Cloning last step.")
+    return(gui_add_step(cohort, changed_input, session))
+  }
+
+  choices <- .available_filters_choices(cohort$get_source(), cohort)
+  selected <- cohort$get_step(cohort$last_step_id())$filters %>%
+    purrr::map_chr("id")
+  if (length(selected) == 0) {
+    selected <- NULL
+  }
+
+  shiny::showModal(
+    shiny::modalDialog(
+      shinyWidgets::virtualSelectInput(
+        ns("manage_step"),
+        label = "Choose filters",
+        choices = choices,
+        selected = selected,
+        multiple = TRUE,
+        html = TRUE,
+        search =  TRUE,
+        selectAllOnlyVisible = TRUE,
+        zIndex = 9999
+      ),
+      shiny::tags$script(
+        shiny::HTML(
+          glue::glue(
+            "$('#{ns('manage_step')}').change(function() {{",
+            "$('#{ns('manage_step_configured')}').attr('disabled', !(this.value.length > 0))}})"
+          )
+        )
+      ),
+      footer = shiny::tagList(
+        shinyGizmo::valueButton(
+          inputId = ns("manage_step_configured"),
+          label = "Accept",
+          selector = paste0("[data-id=\"", ns("manage_step"), "\"]"),
+          onclick = .trigger_action_js("manage_step_configure", ns = ns),
+          `data-dismiss` = "modal", `data-bs-dismiss` = "modal",
+          disabled = NA
+        ),
+        shiny::modalButton("Dismiss")
+      ),
+      title = "Manage last step filters",
+      size = "m",
+      easyClose = TRUE
+    )
+  )
+}
+
+gui_manage_step_configured <- function(cohort, changed_input, session) {
+
+  run_on_request <- !is_none(cohort$attributes$run_button)
+
+  print_state("manage_step", changed_input)
+  input_state("manage_step", changed_input)
+
+  step_id <- cohort$last_step_id()
+  chosen_ids <- session$input[["manage_step"]]
+  current_ids <- cohort$get_filter(step_id) %>% purrr::map_chr("id")
+  to_rm_ids <- setdiff(current_ids, chosen_ids)
+  to_add_ids <- setdiff(chosen_ids, current_ids)
+
+  available_filters <- cohort$attributes$available_filters
+  available_filter_ids <- purrr::map_chr(available_filters, "id")
+  available_filters <- stats::setNames(available_filters, available_filter_ids)
+
+  if (length(available_filters) == 0) {
+    stop("`available_filters` is not defined, configuring step will not be working.")
+  }
+
+  to_add_filters <- available_filters[to_add_ids]
+  for (filter in to_add_filters) {
+    filter_state <- cohortBuilder:::get_filter_state(filter, extra_fields = NULL)
+    cohort$add_filter(
+      filter = do.call(cohortBuilder::filter, filter_state),
+      step_id = step_id,
+      run_flow = FALSE
+    )
+    cohort$update_cache(step_id, filter$id, state = "pre")
+  }
+
+  for (filter_id in to_rm_ids) {
+    cohort$remove_filter(
+      filter_id = filter_id,
+      step_id = step_id,
+      run_flow = FALSE
+    )
+  }
+
+  if (run_on_request) {
+    trigger_pending_state(step_id, "add", session)
+  }
+
+  if (!run_on_request) {
+    cohort$run_step(step_id)
+  }
+
+  for (filter in to_add_filters) {
+    gui_add_step_filter(step_id = step_id, filter_id = filter$id, cohort = cohort, session = session)
+  }
+  for (filter_id in to_rm_ids) {
+    gui_rm_step_filter(step_id = step_id, filter_id = filter_id, cohort = cohort, session = session)
+  }
+  session$sendCustomMessage(
+    "validate_filter_groups",
+    list(step_id = step_id, ns_prefix = session$ns(""))
+  )
+}
+
 gui_run_step <- function(cohort, changed_input, session) {
   ns <- session$ns
 
@@ -494,7 +676,6 @@ gui_run_step <- function(cohort, changed_input, session) {
   input_state("run_step", changed_input)
 
   gui_update_step(cohort, changed_input, session)
-  trigger_pending_state(changed_input$step_id, "remove", session)
 }
 
 gui_show_state <- function(cohort, changed_input, session) {
@@ -656,7 +837,7 @@ gui_show_step_filter_modal <- function(cohort, changed_input, session) {
     shiny::modalDialog(
       shinyWidgets::virtualSelectInput(
         ns("configure_step"),
-        label = "Chose filters",
+        label = "Choose filters",
         choices = choices,
         multiple = TRUE,
         html = TRUE,
@@ -672,7 +853,6 @@ gui_show_step_filter_modal <- function(cohort, changed_input, session) {
           )
         )
       ),
-      easyClose = TRUE,
       footer = shiny::tagList(
         shinyGizmo::valueButton(
           inputId = ns("add_step_configured"),
@@ -686,10 +866,9 @@ gui_show_step_filter_modal <- function(cohort, changed_input, session) {
       ),
       title = "Configure new step",
       size = "m",
-      easyClose = FALSE
+      easyClose = TRUE
     )
   )
-
 }
 
 trigger_pending_state <- function(step_id, action, session) {
@@ -728,6 +907,8 @@ gui_update_step <- function(cohort, changed_input, session) {
   )
   gui_update_data_stats(cohort, list(step_id = changed_input$step_id), session)
 
+  trigger_pending_state(changed_input$step_id, "remove", session)
+
   if (changed_input$run_flow) {
     update_next_step(cohort, changed_input$step_id, FALSE, session)
   }
@@ -1126,7 +1307,9 @@ gui_clear_step <- function(cohort, changed_input, session) {
   print_state("clear_step", changed_input)
   input_state("clear_step", changed_input)
 
-  reset_filters(cohort, changed_input$step_id)
+  if (isTRUE(changed_input$reset)) {
+    reset_filters(cohort, changed_input$step_id)
+  }
 
   if (!is_none(cohort$attributes$run_button)) {
     trigger_pending_state(changed_input$step_id, "add", session)
