@@ -237,7 +237,7 @@ plot_feedback_bar <- function(plot_data, n_missing) {
       feedback_data %>%
       dplyr::mutate(
         tooltip = htmltools::htmlEscape(paste0(level, " (", format_number(n), ")"), TRUE)
-      ) |>
+      ) %>%
       ggplot2::ggplot(
         ggplot2::aes(
           x = "I", y = n, fill = level,
@@ -259,7 +259,7 @@ plot_feedback_bar <- function(plot_data, n_missing) {
         plot.background  = ggplot2::element_blank(),
         legend.position = "none",
         plot.margin = ggplot2::unit(c(0, 0, 0, 0),"mm"),
-        panel.border = ggplot2::element_rect(colour = "grey50", fill = NA, size = 1),
+        panel.border = ggplot2::element_rect(colour = "grey50", fill = NA, linewidth = 1),
         panel.spacing = ggplot2::unit(c(0, 0, 0, 0), "mm")) +
       ggplot2::scale_fill_manual(name = NULL, values = chart_cols) +
       ggiraph::geom_col_interactive(
@@ -353,7 +353,7 @@ plot_feedback_bar <- function(plot_data, n_missing) {
           .trigger_action(session, "update_filter", params = list(
             step_id = filter$step_id, filter_id = filter$id,
             input_name = filter$input_param, input_value = value,
-            run_flow = FALSE
+            update = "force_input", run_flow = FALSE
           ))
         }
       }, ignoreInit = TRUE) %>% .save_observer(input_id, session)
