@@ -1,27 +1,15 @@
-call_filter <- function(filter_id, step_id, cohort, input, output, session) {
+call_filter <- function(filter_id, step_id, cohort, session, feedback) {
   ns <- session$ns
   filter <- cohort$get_filter(step_id, filter_id)
   no_data <- cohort$get_cache(step_id, filter_id, state = "pre")$n_data == 0
-  show_feedback <- if_null_default(
-    filter$get_params("feedback"),
-    cohort$attributes$feedback
-  )
-  if (show_feedback) {
-    feedback <- filter$gui$feedback(sf_id(step_id, filter_id), cohort, no_data)
-  }
 
-  filter$gui$server(sf_id(step_id, filter_id), input, output, session, cohort)
+  filter$gui$server(sf_id(step_id, filter_id), session$input, session$output, session, cohort)
 
-  if (show_feedback) {
+  if (!is.null(feedback)) {
     session$output[[feedback$plot_id]] <- feedback$render_fun
   }
 }
 
-call_filters <- function(step_id, cohort, input, output, session) {
-  cohort$get_filter(step_id, method = names) %>%
-    purrr::walk(~call_filter(.x, step_id, cohort, input, output, session))
-}
-
 #' Create input controller insensitive to server updates
 #'
 #' @description
@@ -89,7 +77,6 @@ call_filters <- function(step_id, cohort, input, output, session) {
 }
 
 render_filter_content <- function(step_filter_id, filter, cohort, ns) {
-  feedback <- filter$gui$feedback(step_filter_id, cohort, NULL)
   cohort$attributes$session$userData$rendered_filters <- c(
     cohort$attributes$session$userData$rendered_filters,
     ns(step_filter_id)
@@ -98,7 +85,29 @@ render_filter_content <- function(step_filter_id, filter, cohort, ns) {
     filter$get_params("feedback"),
     cohort$attributes$feedback
   )
+
+  filter_id <- filter$id
+  step_id <- gsub(paste0("-", filter_id), "", step_filter_id)
+
+  no_data_class <- ""
+  empty <- FALSE
+  if (!cohort$get_cache(step_id, filter_id, state = "pre")$n_data) {
+    no_data_class <- "cb_no_data"
+    empty <- TRUE
+  }
+
+  feedback <- NULL
+  if (show_feedback) {
+    feedback <- filter$gui$feedback(step_filter_id, cohort, empty)
+  }
+
+  call_filter(filter_id, step_id, cohort, cohort$attributes$session, feedback)
+
   shiny::tagList(
+    shiny::div(
+      class = c("cb_no_data_placeholder", no_data_class),
+      "No data in previous step"
+    ),
     if (show_feedback) {
       shiny::div(
         class = "cb_feedback",
@@ -174,11 +183,6 @@ render_filter_content <- function(step_filter_id, filter, cohort, ns) {
   input_param_name <- filter$input_param
   active_filter <- filter$get_params("active")
   filter_description <- cohort$show_help(step_id = step_id, filter_id = filter$id)
-  no_data_class <- if (!cohort$get_cache(step_id, filter_id, state = "pre")$n_data) {
-    "cb_no_data"
-  } else {
-    ""
-  }
   force_render <- getOption("scb_render_all", default = FALSE)
 
   active_id <- paste0("active_", step_filter_id)
@@ -191,14 +195,13 @@ render_filter_content <- function(step_filter_id, filter, cohort, ns) {
       shinyWidgets::prettySwitch(
         inputId = ns(active_id), label = filter$name,
         value = active_filter, inline = TRUE
-      ) |> htmltools::tagAppendAttributes(class = "cb_activate_filter"),
+      ) %>% htmltools::tagAppendAttributes(class = "cb_activate_filter"),
       "active"
     ),
     shiny::div(
       class = paste(
         "cb_filter_content",
-        if (active_filter) NULL else "hidden-input",
-        no_data_class
+        if (active_filter) NULL else "hidden-input"
       ),
       if (active_filter || force_render) {
         render_filter_content(step_filter_id, filter, cohort, ns)
@@ -224,7 +227,7 @@ render_step <- function(cohort, step_id, active, allow_rm, input, output, sessio
       class = "cb_step panel panel-default card",
       header_class = "panel-heading card-header",
       header = shiny::tagList(
-        shiny::tags$strong(class = "cb_step_name", glue::glue("Step {step_id}")),
+        shiny::tags$strong(class = "cb_step_name", glue::glue("{getOption('scb_labels', scb_labels)$step} {step_id}")),
         shiny::tags$div(style = "float: right;",
           button(
             title = getOption("scb_labels", scb_labels)$delete_step_title, class = "cb_rm_step btn-xs",
@@ -274,10 +277,6 @@ render_step <- function(cohort, step_id, active, allow_rm, input, output, sessio
     )
   )
 
-  call_filters(
-    step_id, cohort, input, output, session
-  )
-
   .update_data_stats(cohort$get_source(), step_id, cohort, session)
 }
 
@@ -319,7 +318,7 @@ insert_global_run_button <- function(session) {
     where = "beforeEnd",
     shinyGizmo::conditionalJS(
       condition = htmlwidgets::JS(glue::glue(
-        "$(\'#{ns('cb_steps')} .cb_step:not(.collapsed) .cb_run_step\').prop('disabled')"
+        "scb_is_idle('#{ns('cb_steps')}')"
       )),
       jsCall = shinyGizmo::jsCalls$custom(
         true = htmlwidgets::JS("$(this).prop('disabled', true).addClass('up-to-date');"),
@@ -330,7 +329,7 @@ insert_global_run_button <- function(session) {
         icon = getOption("scb_icons", scb_icons)$run_steps_global,
         disabled = NA,
         onclick = htmlwidgets::JS(glue::glue(
-          "$(\'#{ns('cb_steps')} .cb_step:not(.collapsed) .cb_run_step\').click();"
+          "click_first_busy('#{ns('cb_steps')}');"
         ))
       )
     ),
@@ -377,6 +376,8 @@ render_steps <- function(cohort, session, init = TRUE) {
         add_step_modal = gui_show_step_filter_modal,
         add_step_configure = gui_add_step_configured,
         rm_step = gui_rm_step,
+        manage_step_modal = gui_manage_step_modal,
+        manage_step_configure = gui_manage_step_configured,
         clear_step = gui_clear_step,
         update_step = gui_update_step,
         update_data_stats = gui_update_data_stats,
@@ -674,6 +675,8 @@ restore_attribute <- function(cohort, attribute, value) {
 #' @inheritParams demo_app
 #' @param id Id of the module used to render the panel.
 #' @param ... Extra attributes passed to the panel div container.
+#' @param manage_step When `TRUE`, enables feature, that alows to modify the latest step filters (add/remove them).
+#'   Available list of filters used by the feature should be stores as `source$attributes$available_filters` object.
 #' @return Nested list of `shiny.tag` objects - html structure of filtering panel module.
 #'
 #' @examples
@@ -717,14 +720,73 @@ restore_attribute <- function(cohort, attribute, value) {
 #'
 #'   shinyApp(ui, server)
 #' }
+#' if (interactive()) {
+#'   # enabling latest step filters management
+#'   library(cohortBuilder)
+#'   library(shiny)
+#'   library(shinyCohortBuilder)
 #'
+#'   librarian_source <- set_source(as.tblist(librarian))
+#'   librarian_source$attributes$available_filters <- list(
+#'     filter(
+#'       "discrete", id = "author", dataset = "books",
+#'       variable = "author", value = "Dan Brown",
+#'       active = FALSE
+#'     ),
+#'     filter(
+#'       "range", id = "copies", dataset = "books",
+#'       variable = "copies", range = c(5, 10),
+#'       active = FALSE
+#'     ),
+#'     filter(
+#'       "date_range", id = "registered", dataset = "borrowers",
+#'       variable = "registered", range = c(as.Date("2010-01-01"), Inf),
+#'       active = FALSE
+#'     ),
+#'     filter(
+#'       "discrete", id = "program", dataset = "borrowers",
+#'       variable = "program", value = NA,
+#'       active = FALSE
+#'     )
+#'   )
+#'   librarian_cohort <- cohort(
+#'     librarian_source,
+#'     filter(
+#'       "range", id = "copies", dataset = "books",
+#'       variable = "copies", range = c(5, 10),
+#'       active = FALSE
+#'     ),
+#'     filter(
+#'       "date_range", id = "registered", dataset = "borrowers",
+#'       variable = "registered", range = c(as.Date("2010-01-01"), Inf),
+#'       active = FALSE
+#'     )
+#'   )
+#'
+#'   ui <- fluidPage(
+#'     sidebarLayout(
+#'       sidebarPanel(
+#'         cb_ui("librarian", manage_step = TRUE)
+#'       ),
+#'       mainPanel()
+#'     )
+#'   )
+#'
+#'   server <- function(input, output, session) {
+#'     cb_server("librarian", librarian_cohort)
+#'   }
+#'
+#'   shinyApp(ui, server)
+#' }
 #' @export
-cb_ui <- function(id, ..., state = FALSE, steps = TRUE, code = TRUE, attrition = TRUE, new_step = c("clone", "configure")) {
+cb_ui <- function(id, ..., state = FALSE, steps = TRUE, code = TRUE, attrition = TRUE,
+                  new_step = c("clone", "configure"), manage_step = FALSE) {
   ns <- shiny::NS(id)
   no_steps_class <- if (steps) "" else "cb_no_steps"
   no_state_class <- if (state) "" else "cb_no_state"
   no_code_class <- if (code) "" else "cb_no_code"
   no_attrition_class <- if (attrition) "" else "cb_no_attrition"
+  no_manage_step <- if (manage_step) "" else "cb_no_manage_step"
 
   new_step <- rlang::arg_match(new_step)
   add_step_action <- switch(new_step,
@@ -792,9 +854,16 @@ cb_ui <- function(id, ..., state = FALSE, steps = TRUE, code = TRUE, attrition =
           class = paste(no_attrition_class, "btn-sm")
         ),
         button(
-          getOption("scb_labels", scb_labels)$add_step, class = "cb_add_step btn-sm",
+          getOption("scb_labels", scb_labels)$add_step,
           icon = getOption("scb_icons", scb_icons)$add_step,
-          onclick = .trigger_action_js(add_step_action, ns = ns)
+          onclick = .trigger_action_js(add_step_action, ns = ns),
+          class = c("cb_add_step", "btn-sm")
+        ),
+        button(
+          getOption("scb_labels", scb_labels)$manage_step,
+          icon = getOption("scb_icons", scb_icons)$manage_step,
+          onclick = .trigger_action_js("manage_step_modal", ns = ns),
+          class = c(no_manage_step, "btn-sm"),
         )
       ),
       shinyGizmo::accordion(
@@ -869,6 +938,7 @@ cb_server <- function(id, cohort, run_button = "none", stats = c("pre", "post"),
       }
 
       shiny::onStop(function() {
+
         cohort$attributes$session <- NULL
         cohort$attributes$run_button <- NULL
         cohort$attributes$stats <- NULL
